class Response(object):

    """Response object contains the response returned by the client"""

    def __init__(self, body, code, headers):
        self.body = body
        self.code = code
        self.headers = headers



import six

class Quickemailverification(object):

    """QuickEmailVerification Class for email verification
    """

    def __init__(self, client):
        self.client = client

    def verify(self, email, options={}):
        """Verify email address and get detailed response

        '/verify?email=:email' GET

        Args:
            email: send email address in query parameter
        """
        body = options['query'] if 'query' in options else {}

        email   = six.moves.urllib.parse.quote(email)
        response = self.client.get('/verify?email=' + email + '', body, options)

        return response




class AuthHandler(object):

    """AuthHandler takes care of devising the auth type and using it"""

    HTTP_HEADER = 1

    def __init__(self, auth):
        self.auth = auth

    def get_auth_type(self):
        """Calculating the Authentication Type"""

        if 'http_header' in self.auth:
            return self.HTTP_HEADER

        return -1

    def set(self, request):
        if len(self.auth.keys()) == 0:
            raise StandardError("Server requires authentication to proceed further. Please check")

        auth = self.get_auth_type()
        flag = False

        if auth == self.HTTP_HEADER:
            request = self.http_header(request)
            flag = True

        if not flag:
            raise StandardError("Unable to calculate authorization method. Please check")

        return request

    def http_header(self, request):
        """Authorization with HTTP header"""
        request['headers']['Authorization'] = 'token ' + self.auth['http_header']
        return request




class ResponseHandler(object):

    """ResponseHandler takes care of decoding the response body into suitable type"""

    @staticmethod
    def get_body(response):
        typ = response.headers.get('content-type')
        body = response.text

        # Response body is in JSON
        if typ.find('json') != -1:
            body = response.json()

        return body



from ..error import ClientError
from .response_handler import ResponseHandler


class ErrorHandler(object):

    """ErrorHandler takes care of getting the error message from response body"""

    @staticmethod
    def check_error(response, *args, **kwargs):
        code = response.status_code
        typ = response.headers.get('content-type')

        if code in range(500, 600):
            raise ClientError('Error ' + str(code), code)
        elif code in range(400, 500):
            body = ResponseHandler.get_body(response)
            message = ''

            # If HTML, whole body is taken
            if isinstance(body, str):
                message = body

            # If JSON, a particular field is taken and used
            if typ.find('json') != -1 and isinstance(body, dict):
                if 'message' in body:
                    message = body['message']
                else:
                    message = 'Unable to select error message from json returned by request responsible for error'

            if message == '':
                message = 'Unable to understand the content type of response returned by request responsible for error'

            raise ClientError(message, code)
